--[[
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]

-- local (internal) functions

local function checkAnonymous(name) 
	if(name == nil or name == '') then error('cannot translate anonymous objects') end;
end

local function translatePoint(point, x, y)
	local name, px, py, col = getPoint(point);
	local tx, ty;
	
	if(x == nil) then tx = 0 else tx = x end;
	if(y == nil) then ty = 0 else ty = y end;
	
	setPoint(point, px + tx, py + ty, col);
end

local function translateEllipse(ellipse, x, y)
  local ename, rx, ry = getEllipse(ellipse);
  checkAnonymous(ename);
  translatePoint(ename, x, y);
end

local function translateLine(line, x, y)
	local pname1, pname2 = getLine(line);
	translatePoint(pname1, x, y);
	translatePoint(pname2, x, y);
end

local function translateCurve(curve, x, y)
  local lname, pname = getCurve(curve);
  checkAnonymous(lname);
  checkAnonymous(pname);
  translateLine(lname, x, y);
  translatePoint(pname, x, y);
end

local function translatePoly(poly, x, y)
	for i, line in ipairs({ getPoly(poly) }) do 
		translateLine(line, x, y);
	end
end

local function translateGroup(group, x, y)
	for i, type in ipairs({ getGroup(group) }) do 
		STD_translate(type, x, y);
	end
end

local function listTypesL(name, type)
	local count;
	print('Types in '..name..':');
	count = listTypes(type);
	print('Section total: '..count);
	print('');
	return count;
end

-- public functions (have all to start with 'STD_' by convention)

function STD_displayCopyright()
	print("The standard Lua library for hgl 0.5.42~wheezy -- © 2013-2014 by Heiko Schäfer <heiko@hgl.rangun.de>");
	print("You can report bugs concerning hgl or the standard Lua library to <burmagroup@users.sourceforge.net>");
end

function STD_num2hex(num)
  local hexstr = '0123456789abcdef'
  local s = ''

  while num > 0 do
	local mod = math.fmod(num, 16)
	s = string.sub(hexstr, mod+1, mod+1) .. s
	num = math.floor(num/16)
  end

  if s == '' then s = '0' end

  return string.sub('0'..s, -2);
end

function STD_translate(type, x, y)
	if(typeof(type) == 'LINE') then
		translateLine(type, x, y);
	elseif(typeof(type) == 'POINT') then
		translatePoint(type, x, y)
	elseif(typeof(type) == 'ELLIPSE') then
		translateEllipse(type, x, y);
	elseif(typeof(type) == 'POLY') then
		translatePoly(type, x, y);
	elseif(typeof(type) == 'CURVE') then
		translateCurve(type, x, y);
	elseif(typeof(type) == 'VIRTUALGROUP') then
		translateGroup(type, x, y);
	else
	    error("type not supported: "..type.." ("..typeof(type)..")");
	end
end

function STD_listProceduresTypes()
	return listTypesL('PROCEDURESSECTION', '.procedures');
end

function STD_listObjectsTypes()
    return listTypesL('OBJECTSSECTION', '.objects');
end

function STD_listMainTypes()
	return listTypesL('MAINSECTION', '.main');
end

function STD_listProjectTypes()
	local total = 0;
	total = total + STD_listProceduresTypes();
	total = total + STD_listObjectsTypes();
	total = total + STD_listMainTypes();
	print('Project \''..getProjectName()..'\' total: '..total);
end

function STD_createMultiLine(name, lines, hook)
	if name == nil or lines == nil then
		error("at least one required argument is (nil)");
	end

	for i, line in ipairs(lines) do
		createLine(name..'_line'..i, line[1], line[2], line[3], line[4], line[5], line[6], hook);
	end
end
